#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"

#define DIM 100
#define EPSILON (1e-6)

double penalty = 1000;
int    cost[DIM] = { 500, 70, 93, 25, 500, 30, 20, 10, 10, 5 };
double lamda[DIM] = { 0.45, 0.55, 0.65, 0.75, 0.85, 0.4, 0.3, 0.4, 0.3, 0.2 };


static double factorial(int n) {
    static double save[200];

    assert( 0 <= n );
    assert( n < sizeof(save)/sizeof(save[0]) );
    if (save[n] == 0) {
        int i;
        double r = 1;
        for (i=2; i<=n; i++) r *= i;
        save[n] = r;
    }
    return save[n];
}

static double F(int k, double lamdaj) {
    int m;
    double ret = 0;

    for (m=0; m<=k; m++) {
        ret += exp(-lamdaj) * pow(lamdaj,m) / factorial(m);
    }
    if (ret > 1) ret = 1;  /* pow(lamdaj,m) may be +infinity */
    return ret;
}

static double cache_F(int x, int j) {

    static double save[1000][DIM];

    assert( x < 1000 );
    assert( j < DIM );
    if (save[x][j] == 0) {
        save[x][j] = F( x, lamda[j] );
    }
    return save[x][j];
}


static double func(int dim, int x[]) {
    int j, k;
    double ret = 0;

    assert( dim <= DIM );
    for (j=0; j<dim; j++) {
        if (x[j] < 0) return PLUS_INFINITY;
    }

    for (k=0; /**/; k++) {
        double term = 1;
        for (j=0; j<dim; j++) {
	  term *= cache_F( x[j]+k, j );  /* same that F( x[j]+k, lamda[j] ) */
	  /* term *= F( x[j]+k, lamda[j] ); */
        }
        term = 1 - term;
        ret += term;
        if (term < EPSILON) break;
    }
    ret *= penalty;

    for (j=0; j<dim; j++) {
        ret += cost[j] * x[j];
    }

    /* ivector_print(dim, x);  printf("; func(x)=%g\n", ret); */
    return ret;
}

#ifdef TEST
static void test(int dim) {
    double ans;
    int x[DIM] = {100, 40, 10, 30, 40,10,10,50, 50,50};

    printf("F2(1) = %f\n", F(1, 0.55));
    printf("F2(2) = %f\n", F(2, 0.55));
    printf("F2(3) = %f\n", F(3, 0.55));
    printf("F2(4) = %f\n", F(4, 0.55));

    ans = lgconv_minimize_IFF(dim, func, x);
    if (ans != lgconv_minimize_FW(dim, func, x)) printf("error!\n");
    if (ans != lgconv_minimize(dim, func, x)) printf("error!\n");
}
#endif /* TEST */

static int read_param(int dim) {
    if (!fvector_scan(1, &penalty)) return FALSE;
    if (!ivector_scan(dim, cost))   return FALSE;
    if (!fvector_scan(dim, lamda))  return FALSE;
    return TRUE;
}

void demo_inventory(const int dim) {
    int i, x[DIM];

#ifdef TEST
    for (i=0; i<5; i++) printf("%d! = %g\n", i, factorial(i));
#endif /* TEST */

    if (dim > DIM) {
        fprintf(stderr, "Too many dimension (%d).\n", dim);
        return;
    }

    for (i=0; i<dim; i++) x[i] = 0;
    if (!read_param(dim)) {
        fprintf(stderr, "Read error.\n");
        return;
    }
    
    lgconv_minimize_IFF(dim, func, x);
    /*lgconv_minimize_FW(dim, func, x);*/
}
